@@ -1,7 +1,7 @@
 /*
  * The main header file for SIP.
  *
- * Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2016 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -27,8 +27,8 @@
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x040e02
-#define SIP_VERSION_STR     "4.14.2"
+#define SIP_VERSION         0x041200
+#define SIP_VERSION_STR     "4.18"
 
 
 #ifdef TRUE
@@ -43,9 +43,27 @@
 #define FALSE   0
 
 
+/* Some convenient compiler extensions. */
+
+#if defined(__GNUC__)
+#define SIP_NORETURN    __attribute__((__noreturn__))
+#define SIP_UNUSED      __attribute__((__unused__))
+#elif defined(_MSC_VER)
+#define SIP_NORETURN    __declspec(noreturn)
+#endif
+
+#if !defined(SIP_NORETURN)
+#define SIP_NORETURN
+#endif
+#if !defined(SIP_UNUSED)
+#define SIP_UNUSED
+#endif
+
+
 #define DEFAULT_OFILE_EXT   ".o"    /* Default object file extension. */
 
 #define MAX_NR_ARGS         20      /* Max. nr. args. to a function or template. */
+#define MAX_NR_DEREFS       5       /* Max. nr. type derefences. */
 
 
 /* For convenience. */
@@ -62,6 +80,10 @@
 #define MOD_IS_TRANSFORMED      0x0008  /* It's types have been transformed. */
 #define MOD_USE_ARG_NAMES       0x0010  /* Use real argument names. */
 #define MOD_ALL_RAISE_PY_EXC    0x0020  /* All callable raise a Python exception. */
+#define MOD_SUPER_INIT_NO       0x0040  /* Don't call super().__init__(). */
+#define MOD_SUPER_INIT_YES      0x0080  /* Call super().__init__(). */
+#define MOD_SUPER_INIT_UNDEF    0x0000  /* Calling super().__init__() is undefined. */
+#define MOD_SUPER_INIT_MASK     0x00c0  /* The mask for the above flags. */
 
 #define hasDelayedDtors(m)  ((m)->modflags & MOD_HAS_DELAYED_DTORS)
 #define setHasDelayedDtors(m)   ((m)->modflags |= MOD_HAS_DELAYED_DTORS)
@@ -76,6 +98,10 @@
 #define useArgNames(m)      ((m)->modflags & MOD_USE_ARG_NAMES)
 #define setAllRaisePyException(m)   ((m)->modflags |= MOD_ALL_RAISE_PY_EXC)
 #define allRaisePyException(m)  ((m)->modflags & MOD_ALL_RAISE_PY_EXC)
+#define setCallSuperInitNo(m)   ((m)->modflags = ((m)->modflags & ~MOD_SUPER_INIT_MASK) | MOD_SUPER_INIT_NO)
+#define setCallSuperInitYes(m)  ((m)->modflags = ((m)->modflags & ~MOD_SUPER_INIT_MASK) | MOD_SUPER_INIT_YES)
+#define isCallSuperInitYes(m)   (((m)->modflags & MOD_SUPER_INIT_MASK) == MOD_SUPER_INIT_YES)
+#define isCallSuperInitUndefined(m) (((m)->modflags & MOD_SUPER_INIT_MASK) == MOD_SUPER_INIT_UNDEF)
 
 
 /* Handle section flags. */
@@ -150,8 +176,8 @@
 #define setIsHoldGILDtor(cd) ((cd)->classflags |= CLASS_DTOR_HOLD_GIL)
 #define assignmentHelper(cd) ((cd)->classflags & CLASS_ASSIGN_HELPER)
 #define setAssignmentHelper(cd) ((cd)->classflags |= CLASS_ASSIGN_HELPER)
-#define noPyQt4QMetaObject(cd)  ((cd)->classflags & CLASS_NO_QMETAOBJECT)
-#define setPyQt4NoQMetaObject(cd)   ((cd)->classflags |= CLASS_NO_QMETAOBJECT)
+#define noPyQtQMetaObject(cd)   ((cd)->classflags & CLASS_NO_QMETAOBJECT)
+#define setPyQtNoQMetaObject(cd)    ((cd)->classflags |= CLASS_NO_QMETAOBJECT)
 #define isTemplateClass(cd) ((cd)->classflags & CLASS_IS_TEMPLATE)
 #define setIsTemplateClass(cd)  ((cd)->classflags |= CLASS_IS_TEMPLATE)
 #define resetIsTemplateClass(cd)    ((cd)->classflags &= ~CLASS_IS_TEMPLATE)
@@ -175,10 +201,20 @@
 
 
 /* Handle the second group of class flags. */
+
 #define CLASS2_TMPL_ARG     0x01        /* The class is a template argument. */
+#define CLASS2_MIXIN        0x02        /* The class is a mixin. */
+#define CLASS2_EXPORT_DERIVED   0x04    /* Export the derived class declaration. */
+#define CLASS2_NEEDS_CAST_FUNC  0x08    /* The class needs a cast function. */
 
 #define isTemplateArg(cd)   ((cd)->classflags2 & CLASS2_TMPL_ARG)
 #define setTemplateArg(cd)  ((cd)->classflags2 |= CLASS2_TMPL_ARG)
+#define isMixin(cd)         ((cd)->classflags2 & CLASS2_MIXIN)
+#define setMixin(cd)        ((cd)->classflags2 |= CLASS2_MIXIN)
+#define isExportDerived(cd) ((cd)->classflags2 & CLASS2_EXPORT_DERIVED)
+#define setExportDerived(cd)    ((cd)->classflags2 |= CLASS2_EXPORT_DERIVED)
+#define needsCastFunction(cd)   ((cd)->classflags2 & CLASS2_NEEDS_CAST_FUNC)
+#define setNeedsCastFunction(cd)    ((cd)->classflags2 |= CLASS2_NEEDS_CAST_FUNC)
 
 
 /* Handle ctor flags.  These are combined with the section flags. */
@@ -216,13 +252,16 @@
 /* Handle member flags. */
 
 #define MEMBR_NUMERIC       0x0001      /* It is a numeric slot. */
-#define MEMBR_NO_ARG_PARSER 0x0002      /* Don't generate an argument parser. */
-#define MEMBR_NOT_VERSIONED 0x0004      /* There is an unversioned overload. */
-#define MEMBR_KEYWORD_ARGS  0x0008      /* It allows keyword arguments. */
-#define MEMBR_HAS_PROTECTED 0x0010      /* It has a protected overload. */
+#define MEMBR_SEQUENCE      0x0002      /* It is a sequnce slot. */
+#define MEMBR_NO_ARG_PARSER 0x0004      /* Don't generate an argument parser. */
+#define MEMBR_NOT_VERSIONED 0x0008      /* There is an unversioned overload. */
+#define MEMBR_KEYWORD_ARGS  0x0010      /* It allows keyword arguments. */
+#define MEMBR_HAS_PROTECTED 0x0011      /* It has a protected overload. */
 
 #define isNumeric(m)        ((m)->memberflags & MEMBR_NUMERIC)
 #define setIsNumeric(m)     ((m)->memberflags |= MEMBR_NUMERIC)
+#define isSequence(m)       ((m)->memberflags & MEMBR_SEQUENCE)
+#define setIsSequence(m)    ((m)->memberflags |= MEMBR_SEQUENCE)
 #define noArgParser(m)      ((m)->memberflags & MEMBR_NO_ARG_PARSER)
 #define setNoArgParser(m)   ((m)->memberflags |= MEMBR_NO_ARG_PARSER)
 #define notVersioned(m)     ((m)->memberflags & MEMBR_NOT_VERSIONED)
@@ -236,6 +275,7 @@
 /* Handle enum flags.  These are combined with the section flags. */
 
 #define ENUM_WAS_PROT       0x00000100  /* It was defined as protected. */
+#define ENUM_NO_SCOPE       0x00000200  /* Omit the member scopes. */
 
 #define isProtectedEnum(e)  ((e)->enumflags & SECT_IS_PROT)
 #define setIsProtectedEnum(e)   ((e)->enumflags |= SECT_IS_PROT)
@@ -244,6 +284,8 @@
 #define wasProtectedEnum(e) ((e)->enumflags & ENUM_WAS_PROT)
 #define setWasProtectedEnum(e)  ((e)->enumflags |= ENUM_WAS_PROT)
 #define resetWasProtectedEnum(e)    ((e)->enumflags &= ~ENUM_WAS_PROT)
+#define isNoScope(e)        ((e)->enumflags & ENUM_NO_SCOPE)
+#define setIsNoScope(e)     ((e)->enumflags |= ENUM_NO_SCOPE)
 
 
 /* Handle hierarchy flags. */
@@ -251,6 +293,7 @@
 #define HIER_IS_DUPLICATE   0x0001      /* It is a super class duplicate. */
 #define HIER_HAS_DUPLICATE  0x0002      /* It has a super class duplicate. */
 #define HIER_BEING_SET      0x0004      /* The MRO is being set. */
+#define HIER_NEEDS_CAST     0x0008      /* The class needs an explicit cast. */
 
 #define isDuplicateSuper(m) ((m)->mroflags & HIER_IS_DUPLICATE)
 #define setIsDuplicateSuper(m)  ((m)->mroflags |= HIER_IS_DUPLICATE)
@@ -259,6 +302,8 @@
 #define hierBeingSet(m)     ((m)->mroflags & HIER_BEING_SET)
 #define setHierBeingSet(m)  ((m)->mroflags |= HIER_BEING_SET)
 #define resetHierBeingSet(m)    ((m)->mroflags &= ~HIER_BEING_SET)
+#define needsCast(m)        ((m)->mroflags & HIER_NEEDS_CAST)
+#define setNeedsCast(m)     ((m)->mroflags |= HIER_NEEDS_CAST)
 
 
 /* Handle overload flags.  These are combined with the section flags. */
@@ -347,32 +392,36 @@
 /* Handle variable flags. */
 
 #define VAR_IS_STATIC       0x01    /* It is a static variable. */
-#define VAR_NEEDS_HANDLER   0x02    /* It the variable needs a handler. */
+#define VAR_NEEDS_HANDLER   0x02    /* The variable needs a handler. */
+#define VAR_NO_SETTER       0x04    /* The variable has no setter. */
 
 #define isStaticVar(v)      ((v)->varflags & VAR_IS_STATIC)
 #define setIsStaticVar(v)   ((v)->varflags |= VAR_IS_STATIC)
 #define needsHandler(v)     ((v)->varflags & VAR_NEEDS_HANDLER)
 #define setNeedsHandler(v)  ((v)->varflags |= VAR_NEEDS_HANDLER)
+#define noSetter(v)         ((v)->varflags & VAR_NO_SETTER)
+#define setNoSetter(v)      ((v)->varflags |= VAR_NO_SETTER)
 
 
 /* Handle argument flags. */
 
-#define ARG_IS_REF          0x0001  /* It is a reference. */
-#define ARG_IS_CONST        0x0002  /* It is a const. */
-#define ARG_XFERRED         0x0004  /* Ownership is transferred. */
-#define ARG_THIS_XFERRED    0x0008  /* Ownership of this is transferred. */
-#define ARG_XFERRED_BACK    0x0010  /* Ownership is transferred back. */
-#define ARG_ARRAY           0x0020  /* Used as an array. */
-#define ARG_ARRAY_SIZE      0x0040  /* Used as an array size. */
-#define ARG_ALLOW_NONE      0x0080  /* Allow None as a value. */
-#define ARG_GET_WRAPPER     0x0100  /* Get the wrapper object. */
-#define ARG_IN              0x0200  /* It passes an argument. */
-#define ARG_OUT             0x0400  /* It returns a result. */
-#define ARG_CONSTRAINED     0x0800  /* Suppress type conversion. */
-#define ARG_SINGLE_SHOT     0x1000  /* The slot is only ever fired once. */
-#define ARG_RESULT_SIZE     0x2000  /* It defines the result size. */
-#define ARG_KEEP_REF        0x4000  /* Keep a reference. */
-#define ARG_NO_COPY         0x8000  /* Disable copying of const references. */
+#define ARG_IS_REF          0x00000001  /* It is a reference. */
+#define ARG_IS_CONST        0x00000002  /* It is a const. */
+#define ARG_XFERRED         0x00000004  /* Ownership is transferred. */
+#define ARG_THIS_XFERRED    0x00000008  /* Ownership of this is transferred. */
+#define ARG_XFERRED_BACK    0x00000010  /* Ownership is transferred back. */
+#define ARG_ARRAY           0x00000020  /* Used as an array. */
+#define ARG_ARRAY_SIZE      0x00000040  /* Used as an array size. */
+#define ARG_ALLOW_NONE      0x00000080  /* Allow None as a value. */
+#define ARG_GET_WRAPPER     0x00000100  /* Get the wrapper object. */
+#define ARG_IN              0x00000200  /* It passes an argument. */
+#define ARG_OUT             0x00000400  /* It returns a result. */
+#define ARG_CONSTRAINED     0x00000800  /* Suppress type conversion. */
+#define ARG_SINGLE_SHOT     0x00001000  /* The slot is only ever fired once. */
+#define ARG_RESULT_SIZE     0x00002000  /* It defines the result size. */
+#define ARG_KEEP_REF        0x00004000  /* Keep a reference. */
+#define ARG_NO_COPY         0x00008000  /* Disable copying of const refs. */
+#define ARG_DISALLOW_NONE   0x00010000  /* Disallow None as a value. */
 
 #define isReference(a)      ((a)->argflags & ARG_IS_REF)
 #define setIsReference(a)   ((a)->argflags |= ARG_IS_REF)
@@ -408,6 +457,8 @@
 #define setKeepReference(a) ((a)->argflags |= ARG_KEEP_REF)
 #define noCopy(a)           ((a)->argflags & ARG_NO_COPY)
 #define setNoCopy(a)        ((a)->argflags |= ARG_NO_COPY)
+#define isDisallowNone(a)   ((a)->argflags & ARG_DISALLOW_NONE)
+#define setDisallowNone(a)  ((a)->argflags |= ARG_DISALLOW_NONE)
 
 
 /* Handle name flags. */
@@ -426,12 +477,15 @@
 
 #define VH_IS_DUPLICATE     0x01    /* It is a duplicate. */
 #define VH_TRANSFERS        0x02    /* It transfers ownership of the result. */
+#define VH_ABORT_ON_EXC     0x04    /* It aborts on an exception. */
 
 #define isDuplicateVH(vh)   ((vh)->vhflags & VH_IS_DUPLICATE)
 #define setIsDuplicateVH(vh)    ((vh)->vhflags |= VH_IS_DUPLICATE)
 #define resetIsDuplicateVH(vh)  ((vh)->vhflags &= ~VH_IS_DUPLICATE)
 #define isTransferVH(vh)    ((vh)->vhflags & VH_TRANSFERS)
 #define setIsTransferVH(vh) ((vh)->vhflags |= VH_TRANSFERS)
+#define abortOnException(vh)    ((vh)->vhflags & VH_ABORT_ON_EXC)
+#define setAbortOnException(vh) ((vh)->vhflags |= VH_ABORT_ON_EXC)
 
 
 /* Handle mapped type flags. */
@@ -478,7 +532,6 @@ typedef enum {
 /* Slot types. */
 typedef enum {
     str_slot,
-    unicode_slot,
     int_slot,
     long_slot,
     float_slot,
@@ -535,6 +588,11 @@ typedef enum {
     next_slot,
     setattr_slot,
     delattr_slot,       /* This is local to the parser. */
+    matmul_slot,
+    imatmul_slot,
+    await_slot,
+    aiter_slot,
+    anext_slot,
     no_slot
 } slotType;
 
@@ -602,7 +660,6 @@ typedef enum {
 
 
 /* Value types. */
-
 typedef enum {
     qchar_value,
     string_value,
@@ -614,7 +671,6 @@ typedef enum {
 
 
 /* Version types. */
-
 typedef enum {
     time_qualifier,
     platform_qualifier,
@@ -623,7 +679,6 @@ typedef enum {
 
 
 /* Interface file types. */
-
 typedef enum {
     exception_iface,
     mappedtype_iface,
@@ -632,6 +687,24 @@ typedef enum {
 } ifaceFileType;
 
 
+/* Type hint parse status. */
+typedef enum {
+    needs_parsing,
+    being_parsed,
+    parsed
+} typeHintParseStatus;
+
+
+/* Type hint node type. */
+typedef enum {
+    typing_node,
+    class_node,
+    enum_node,
+    brackets_node,
+    other_node
+} typeHintNodeType;
+
+
 /* A location in a .sip source file. */
 typedef struct {
     int linenr;                         /* The line number. */
@@ -640,7 +713,6 @@ typedef struct {
 
 
 /* A software license. */
-
 typedef struct {
     const char *type;                   /* The license type. */
     const char *licensee;               /* The licensee. */
@@ -650,19 +722,18 @@ typedef struct {
 
 
 /* A version qualifier. */
-
 typedef struct _qualDef {
     const char *name;                   /* The qualifier name. */
     qualType qtype;                     /* The qualifier type. */
     struct _moduleDef *module;          /* The defining module. */
     int line;                           /* Timeline if it is a time. */
     int order;                          /* Order if it is a time. */
+    int default_enabled;                /* Enabled by default. */
     struct _qualDef *next;              /* Next in the list. */
 } qualDef;
 
 
 /* A scoped name. */
-
 typedef struct _scopedNameDef {
     char *name;                         /* The name. */
     struct _scopedNameDef *next;        /* Next in the scope list. */
@@ -670,7 +741,6 @@ typedef struct _scopedNameDef {
 
 
 /* A name. */
-
 typedef struct _nameDef {
     int nameflags;                      /* The name flags. */
     const char *text;                   /* The text of the name. */
@@ -681,7 +751,6 @@ typedef struct _nameDef {
 
 
 /* A literal code block. */
-
 typedef struct _codeBlock {
     char *frag;                         /* The code itself. */
     const char *filename;               /* The original file. */
@@ -690,7 +759,6 @@ typedef struct _codeBlock {
 
 
 /* A list of literal code blocks. */
-
 typedef struct _codeBlockList {
     codeBlock *block;                   /* The code block. */
     struct _codeBlockList *next;        /* The next in the list. */
@@ -698,7 +766,6 @@ typedef struct _codeBlockList {
 
 
 /* The arguments to a throw specifier. */
-
 typedef struct _throwArgs {
     int nrArgs;                         /* The number of arguments. */
     struct _exceptionDef *args[MAX_NR_ARGS];    /* The arguments. */
@@ -706,7 +773,6 @@ typedef struct _throwArgs {
 
 
 /* An exception. */
-
 typedef struct _exceptionDef {
     int exceptionnr;                    /* The exception number. */
     struct _ifaceFileDef *iff;          /* The interface file. */
@@ -720,11 +786,11 @@ typedef struct _exceptionDef {
 
 
 /* A value. */
-
 typedef struct _valueDef {
     valueType vtype;                    /* The type. */
     char vunop;                         /* Any unary operator. */
     char vbinop;                        /* Any binary operator. */
+    scopedNameDef *cast;                /* Any cast. */
     union {
         char vqchar;                    /* Quoted character value. */
         long vnum;                      /* Numeric value. */
@@ -738,15 +804,17 @@ typedef struct _valueDef {
 
 
 /* A member function argument (or result). */
-
 typedef struct {
     argType atype;                      /* The type. */
     nameDef *name;                      /* The name. */
     const char *doctype;                /* The documented type. */
+    struct _typeHintDef *typehint_in;   /* The PEP 484 input type hint. */
+    struct _typeHintDef *typehint_out;  /* The PEP 484 output type hint. */
+    const char *typehint_value;         /* The type hint value. */
     int argflags;                       /* The argument flags. */
     int nrderefs;                       /* Nr. of dereferences. */
+    int derefs[MAX_NR_DEREFS];          /* The const for each dereference. */
     valueDef *defval;                   /* The default value. */
-    const char *docval;                 /* The documented value. */
     int key;                            /* The optional /KeepReference/ key. */
     struct _typedefDef *original_type;  /* The original type if typedef'd. */
     union {
@@ -797,6 +865,27 @@ typedef struct _virtErrorHandler {
 } virtErrorHandler;
 
 
+/* A parsed PEP 484 compliant type hint. */
+typedef struct _typeHintDef {
+    typeHintParseStatus status;         /* The state of the type hint parse. */
+    char *raw_hint;                     /* The raw hint. */
+    struct _typeHintNodeDef *root;      /* The root of parsed nodes. */
+} typeHintDef;
+
+
+/* A node of a parsed type hint. */
+typedef struct _typeHintNodeDef {
+    typeHintNodeType type;              /* The type of the node. */
+    union {
+        const char *name;               /* For typing objects and others. */
+        struct _classDef *cd;           /* For class nodes. */
+        struct _enumDef *ed;            /* For enum nodes. */
+    } u;
+    struct _typeHintNodeDef *children;  /* The list of children. */
+    struct _typeHintNodeDef *next;      /* The next sibling. */
+} typeHintNodeDef;
+
+
 /* A module definition. */
 typedef struct _moduleDef {
     nameDef *fullname;                  /* The full module name. */
@@ -805,8 +894,7 @@ typedef struct _moduleDef {
     apiVersionRangeDef *api_versions;   /* The defined APIs. */
     apiVersionRangeDef *api_ranges;     /* The list of API version ranges. */
     int modflags;                       /* The module flags. */
-    KwArgs kwargs;                      /* The styleof keyword argument support. */
-    int qobjclass;                      /* QObject class, -1 if none. */
+    KwArgs kwargs;                      /* The style of keyword argument support. */
     struct _memberDef *othfuncs;        /* List of other functions. */
     struct _overDef *overs;             /* Global overloads. */
     Format defdocstring;                /* The default docstring format. */
@@ -823,6 +911,7 @@ typedef struct _moduleDef {
     codeBlockList *unitcode;            /* Compilation unit code. */
     codeBlockList *unitpostinccode;     /* Compilation unit post-include code. */
     codeBlockList *docstring;           /* The docstring. */
+    codeBlockList *typehintcode;        /* Type hint code. */
     const char *virt_error_handler;     /* The virtual error handler. */
     int parts;                          /* The number of parts generated. */
     const char *file;                   /* The filename. */
@@ -855,7 +944,6 @@ typedef struct _moduleListDef {
 
 
 /* An interface file definition. */
-
 typedef struct _ifaceFileDef {
     nameDef *name;                      /* The name. */
     apiVersionRangeDef *api_range;      /* The optional API version range. */
@@ -866,6 +954,7 @@ typedef struct _ifaceFileDef {
     scopedNameDef *fqcname;             /* The fully qualified C++ name. */
     moduleDef *module;                  /* The owning module. */
     codeBlockList *hdrcode;             /* Header code. */
+    const char *file_extension;         /* The optional file extension. */
     struct _ifaceFileList *used;        /* Interface files used. */
     struct _ifaceFileDef *next;         /* Next in the list. */
 } ifaceFileDef;
@@ -880,13 +969,15 @@ typedef struct _ifaceFileList {
 
 
 /* A mapped type. */
-
 typedef struct _mappedTypeDef {
     int mtflags;                        /* The mapped type flags. */
     argDef type;                        /* The type being mapped. */
     nameDef *pyname;                    /* The Python name. */
     nameDef *cname;                     /* The C/C++ name. */
     const char *doctype;                /* The documented type. */
+    typeHintDef *typehint_in;           /* The PEP 484 input type hint. */
+    typeHintDef *typehint_out;          /* The PEP 484 output type hint. */
+    const char *typehint_value;         /* The type hint value. */
     ifaceFileDef *iff;                  /* The interface file. */
     struct _memberDef *members;         /* The static member functions. */
     struct _overDef *overs;             /* The static overloads. */
@@ -899,7 +990,6 @@ typedef struct _mappedTypeDef {
 
 
 /* A function signature. */
-
 typedef struct _signatureDef {
     argDef result;                      /* The result. */
     int nrArgs;                         /* The number of arguments. */
@@ -908,7 +998,6 @@ typedef struct _signatureDef {
 
 
 /* A list of function signatures. */
-
 typedef struct _signatureList {
     struct _signatureDef *sd;           /* The signature. */
     struct _signatureList *next;        /* Next in the list. */
@@ -916,7 +1005,6 @@ typedef struct _signatureList {
 
 
 /* A template type. */
-
 typedef struct _templateDef {
     scopedNameDef *fqname;              /* The name. */
     signatureDef types;                 /* The types. */
@@ -924,7 +1012,6 @@ typedef struct _templateDef {
 
 
 /* A list of virtual handlers. */
-
 typedef struct _virtHandlerDef {
     int virthandlernr;                  /* The nr. of the virtual handler. */
     int vhflags;                        /* The virtual handler flags. */
@@ -937,7 +1024,6 @@ typedef struct _virtHandlerDef {
 
 
 /* A typedef definition. */
-
 typedef struct _typedefDef {
     int tdflags;                        /* The typedef flags. */
     scopedNameDef *fqname;              /* The fully qualified name. */
@@ -949,10 +1035,10 @@ typedef struct _typedefDef {
 
 
 /* A variable definition. */
-
 typedef struct _varDef {
-    nameDef *pyname;                    /* The variable Python name. */
     scopedNameDef *fqcname;             /* The fully qualified C/C++ name. */
+    nameDef *pyname;                    /* The variable Python name. */
+    int no_typehint;                    /* The type hint will be suppressed. */
     struct _classDef *ecd;              /* The enclosing class. */
     moduleDef *module;                  /* The owning module. */
     int varflags;                       /* The variable flags. */
@@ -965,7 +1051,6 @@ typedef struct _varDef {
 
 
 /* A property definition. */
-
 typedef struct _propertyDef {
     nameDef *name;                      /* The property name. */
     const char *get;                    /* The name of the getter method. */
@@ -976,11 +1061,12 @@ typedef struct _propertyDef {
 
 
 /* An overloaded member function definition. */
-
 typedef struct _overDef {
     sourceLocation sloc;                /* The source location. */
     char *cppname;                      /* The C++ name. */
     int overflags;                      /* The overload flags. */
+    int no_typehint;                    /* The type hint will be suppressed. */
+    int pyqt_signal_hack;               /* The PyQt signal hack. */
     KwArgs kwargs;                      /* The keyword argument support. */
     struct _memberDef *common;          /* Common parts. */
     apiVersionRangeDef *api_range;      /* The optional API version range. */
@@ -988,6 +1074,7 @@ typedef struct _overDef {
     signatureDef *cppsig;               /* The C++ signature. */
     throwArgs *exceptions;              /* The exceptions. */
     codeBlockList *methodcode;          /* Method code. */
+    codeBlockList *virtcallcode;        /* Virtual call code. */
     virtHandlerDef *virthandler;        /* The virtual handler. */
     char *prehook;                      /* The pre-hook name. */
     char *posthook;                     /* The post-hook name. */
@@ -997,9 +1084,9 @@ typedef struct _overDef {
 
 
 /* An overloaded constructor definition. */
-
 typedef struct _ctorDef {
     int ctorflags;                      /* The ctor flags. */
+    int no_typehint;                    /* The type hint will be suppressed. */
     KwArgs kwargs;                      /* The keyword argument support. */
     apiVersionRangeDef *api_range;      /* The optional API version range. */
     signatureDef pysig;                 /* The Python signature. */
@@ -1013,9 +1100,9 @@ typedef struct _ctorDef {
 
 
 /* An enumerated type member definition. */
-
 typedef struct _enumMemberDef {
     nameDef *pyname;                    /* The Python name. */
+    int no_typehint;                    /* The type hint will be suppressed. */
     char *cname;                        /* The C/C++ name. */
     struct _enumDef *ed;                /* The enclosing enum. */
     struct _enumMemberDef *next;        /* Next in the list. */
@@ -1023,12 +1110,12 @@ typedef struct _enumMemberDef {
 
 
 /* An enumerated type definition. */
-
 typedef struct _enumDef {
     int enumflags;                      /* The enum flags. */
-    nameDef *pyname;                    /* The Python name (may be NULL). */
     scopedNameDef *fqcname;             /* The C/C++ name (may be NULL). */
     nameDef *cname;                     /* The C/C++ name (may be NULL). */
+    nameDef *pyname;                    /* The Python name (may be NULL). */
+    int no_typehint;                    /* The type hint will be suppressed. */
     struct _enumDef *first_alt;         /* The first alternate API. */
     struct _enumDef *next_alt;          /* The next alternate API. */
     int enumnr;                         /* The enum number. */
@@ -1044,7 +1131,6 @@ typedef struct _enumDef {
 
 
 /* An member function definition. */
-
 typedef struct _memberDef {
     nameDef *pyname;                    /* The Python name. */
     int memberflags;                    /* The member flags. */
@@ -1057,7 +1143,6 @@ typedef struct _memberDef {
 
 
 /* A list of visible member functions. */
-
 typedef struct _visibleList {
     memberDef *m;                       /* The member definition. */
     struct _classDef *cd;               /* The class. */
@@ -1066,7 +1151,6 @@ typedef struct _visibleList {
 
 
 /* An entry in a linked class list. */
-
 typedef struct _classList {
     struct _classDef *cd;               /* The class itself. */
     struct _classList *next;            /* Next in the list. */
@@ -1074,16 +1158,13 @@ typedef struct _classList {
 
 
 /* A virtual overload definition. */
-
 typedef struct _virtOverDef {
     overDef o;                          /* The overload. */
-    struct _classDef *scope;            /* The overload scope. */
     struct _virtOverDef *next;          /* Next in the list. */
 } virtOverDef;
 
 
 /* A class that appears in a class's hierarchy. */
-
 typedef struct _mroDef {
     struct _classDef *cd;               /* The class. */
     int mroflags;                       /* The hierarchy flags. */
@@ -1092,12 +1173,13 @@ typedef struct _mroDef {
 
 
 /* A class definition. */
-
 typedef struct _classDef {
     unsigned classflags;                /* The class flags. */
     unsigned classflags2;               /* The class flags, part 2. */
-    int pyqt4_flags;                    /* The PyQt4 specific flags. */
+    int pyqt_flags;                     /* The PyQt specific flags. */
+    const char *pyqt_interface;         /* The Qt interface name. */
     nameDef *pyname;                    /* The Python name. */
+    int no_typehint;                    /* The type hint will be suppressed. */
     ifaceFileDef *iff;                  /* The interface file. */
     struct _classDef *ecd;              /* The enclosing scope. */
     struct _classDef *real;             /* The real class if this is a proxy or extender. */
@@ -1122,6 +1204,7 @@ typedef struct _classDef {
     codeBlockList *docstring;           /* Class and ctor docstrings. */
     codeBlockList *instancecode;        /* Create instance code. */
     codeBlockList *convtocode;          /* Convert to C++ code. */
+    codeBlockList *convfromcode;        /* Convert from C++ code. */
     codeBlockList *travcode;            /* Traverse code. */
     codeBlockList *clearcode;           /* Clear code. */
     codeBlockList *getbufcode;          /* Get buffer code (Python v3). */
@@ -1131,14 +1214,18 @@ typedef struct _classDef {
     codeBlockList *segcountcode;        /* Segment count code (Python v2). */
     codeBlockList *charbufcode;         /* Character buffer code (Python v2). */
     codeBlockList *picklecode;          /* Pickle code. */
+    codeBlockList *finalcode;           /* Finalisation code. */
+    codeBlockList *typehintcode;        /* Type hint code. */
     propertyDef *properties;            /* The properties. */
     const char *virt_error_handler;     /* The virtual error handler. */
+    typeHintDef *typehint_in;           /* The PEP 484 input type hint. */
+    typeHintDef *typehint_out;          /* The PEP 484 output type hint. */
+    const char *typehint_value;         /* The type hint value. */
     struct _classDef *next;             /* Next in the list. */
 } classDef;
 
 
 /* A class template definition. */
-
 typedef struct _classTmplDef {
     signatureDef sig;                   /* The template arguments. */
     classDef *cd;                       /* The class itself. */
@@ -1147,7 +1234,6 @@ typedef struct _classTmplDef {
 
 
 /* A mapped type template definition. */
-
 typedef struct _mappedTypeTmplDef {
     signatureDef sig;                   /* The template arguments. */
     mappedTypeDef *mt;                  /* The mapped type itself. */
@@ -1156,7 +1242,6 @@ typedef struct _mappedTypeTmplDef {
 
 
 /* The extracts for an identifier. */
-
 typedef struct _extractDef {
     const char *id;                     /* The identifier. */
     struct _extractPartDef *parts;      /* The ordered list of parts. */
@@ -1165,7 +1250,6 @@ typedef struct _extractDef {
 
 
 /* Part of an extract for an identifier. */
-
 typedef struct _extractPartDef {
     int order;                          /* The order of the part. */
     codeBlock *part;                    /* The part itself. */
@@ -1174,7 +1258,6 @@ typedef struct _extractPartDef {
 
 
 /* A rule for automatic Python naming. */
-
 typedef struct _autoPyNameDef {
     const char *remove_leading;         /* Leading string to remove. */
     struct _autoPyNameDef *next;        /* The next in the list. */
@@ -1182,7 +1265,6 @@ typedef struct _autoPyNameDef {
 
 
 /* The parse tree corresponding to the specification file. */
-
 typedef struct {
     moduleDef *module;                  /* The module being generated. */
     moduleDef *modules;                 /* The list of modules. */
@@ -1198,7 +1280,9 @@ typedef struct {
     typedefDef *typedefs;               /* List of typedefs. */
     virtErrorHandler *errorhandlers;    /* The list of virtual error handlers. */
     codeBlockList *exphdrcode;          /* Exported header code. */
+    codeBlockList *exptypehintcode;     /* Exported type hint code. */
     codeBlockList *docs;                /* Documentation. */
+    classDef *qobject_cd;               /* QObject class, NULL if none. */
     int sigslots;                       /* Set if signals or slots are used. */
     int genc;                           /* Set if we are generating C code. */
     struct _stringList *plugins;        /* The list of plugins. */
@@ -1207,7 +1291,6 @@ typedef struct {
 
 
 /* A list of strings. */
-
 typedef struct _stringList {
     const char *s;                      /* The string. */
     struct _stringList *next;           /* The next in the list. */
@@ -1215,7 +1298,6 @@ typedef struct _stringList {
 
 
 /* File specific context information for the parser. */
-
 typedef struct _parserContext {
     const char *filename;               /* The %Import or %Include filename. */
     int ifdepth;                        /* The depth of nested if's. */
@@ -1227,7 +1309,8 @@ extern char *sipVersion;                /* The version of SIP. */
 extern stringList *includeDirList;      /* The include directory list for SIP files. */
 
 
-void parse(sipSpec *, FILE *, char *, stringList *, stringList *, KwArgs, int);
+void parse(sipSpec *, FILE *, char *, stringList *, stringList *, stringList *,
+        KwArgs, int);
 void parserEOF(const char *,parserContext *);
 void transform(sipSpec *);
 void generateCode(sipSpec *, char *, char *, char *, const char *, int, int,
@@ -1237,11 +1320,13 @@ void generateExtracts(sipSpec *pt, const stringList *extracts);
 void addExtractPart(sipSpec *pt, const char *id, int order, codeBlock *part);
 void generateAPI(sipSpec *pt, moduleDef *mod, const char *apiFile);
 void generateXML(sipSpec *pt, moduleDef *mod, const char *xmlFile);
+void generateTypeHints(sipSpec *pt, moduleDef *mod, const char *pyiFile);
 void generateExpression(valueDef *vd, int in_str, FILE *fp);
 void warning(Warning w, const char *fmt, ...);
 void deprecated(const char *msg);
-void fatal(char *,...);
+SIP_NORETURN void fatal(const char *fmt,...);
 void fatalScopedName(scopedNameDef *);
+void fatalStart();
 void getSourceLocation(sourceLocation *slp);
 int setInputFile(FILE *open_fp, parserContext *pc, int optional);
 void resetLexerState();
@@ -1250,7 +1335,7 @@ void *sipCalloc(size_t nr, size_t n);
 char *sipStrdup(const char *);
 char *concat(const char *, ...);
 void append(char **, const char *);
-void addToUsedList(ifaceFileList **, ifaceFileDef *);
+void appendToIfaceFileList(ifaceFileList **ifflp, ifaceFileDef *iff);
 int selectedQualifier(stringList *needed_qualifiers, qualDef *qd);
 int excludedFeature(stringList *,qualDef *);
 int sameSignature(signatureDef *,signatureDef *,int);
@@ -1265,11 +1350,11 @@ void freeScopedName(scopedNameDef *);
 void appendToClassList(classList **,classDef *);
 void appendCodeBlockList(codeBlockList **headp, codeBlockList *cbl);
 void prcode(FILE *fp, const char *fmt, ...);
+void prCopying(FILE *fp, moduleDef *mod, const char *comment);
 void prOverloadName(FILE *fp, overDef *od);
 void prOverloadDecl(FILE *fp, ifaceFileDef *scope, overDef *od, int defval);
+void prDefaultValue(argDef *ad, int in_str, FILE *fp);
 void prScopedPythonName(FILE *fp, classDef *scope, const char *pyname);
-int prPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd, int sec,
-        int names, int defaults, int in_str, int is_signal);
 void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad);
 int isZeroArgSlot(memberDef *md);
 int isIntReturnSlot(memberDef *md);
@@ -1289,11 +1374,24 @@ ifaceFileDef *findIfaceFile(sipSpec *pt, moduleDef *mod,
         apiVersionRangeDef *api_range, argDef *ad);
 int pluginPyQt3(sipSpec *pt);
 int pluginPyQt4(sipSpec *pt);
+int pluginPyQt5(sipSpec *pt);
+SIP_NORETURN void yyerror(char *);
 void yywarning(char *);
+int yylex();
 nameDef *cacheName(sipSpec *pt, const char *name);
 scopedNameDef *encodedTemplateName(templateDef *td);
 apiVersionRangeDef *findAPI(sipSpec *pt, const char *name);
 memberDef *findMethod(classDef *cd, const char *name);
+typeHintDef *newTypeHint(char *raw_hint);
+int isPyKeyword(const char *word);
+void getDefaultImplementation(sipSpec *pt, argType atype, classDef **cdp,
+        mappedTypeDef **mtdp);
+char *templateString(const char *src, scopedNameDef *names,
+        scopedNameDef *values);
+int inDefaultAPI(sipSpec *pt, apiVersionRangeDef *range);
+int hasImplicitOverloads(signatureDef *sd);
+void dsCtor(sipSpec *pt, classDef *cd, ctorDef *ct, int sec, FILE *fp);
+void dsOverload(sipSpec *pt, overDef *od, int is_method, int sec, FILE *fp);
 
 
 /* These are only here because bison publically references them. */
@@ -1438,6 +1536,7 @@ typedef struct _moduleCfg {
     const char *name;
     int use_arg_names;
     int all_raise_py_exc;
+    int call_super_init;
     const char *def_error_handler;
     int version;
     codeBlock *docstring;
